package com.btc.ep.dal.persist.impl;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;

import com.btc.ep.dal.persist.PersistenceManager;
import com.btc.ep.dal.provider.PersistenceManagerProvider;
import com.btc.ep.dal.provider.impl.OdbPersistenceManagerProvider;

public class PersistenceManagerImpl extends PersistenceManager {

    public static final String DATABASE_ARG = "epp.database";

    private static final String DEFAULT_DB = OdbPersistenceManagerProvider.ID;

    private PersistenceService service;

    private Map<String, PersistenceManagerProvider> providers = new HashMap<>();

    private PersistenceManagerProvider provider;

    public PersistenceManagerImpl() {

    }

    protected void activate() {
    }

    public void addProvider(PersistenceManagerProvider provider) {
        providers.put(provider.getId(), provider);
    }

    public void removeProvider(PersistenceManagerProvider provider) {
        providers.remove(provider.getId());
    }

    @Override
    public PersistenceService getPersistenceService() {
        getProvider();

        if (service == null) {
            EntityManagerFactory emFactory = provider.getEntityManagerFactory();
            this.service = new PersistenceService(emFactory.createEntityManager());
        }

        return service;
    }

    protected PersistenceManagerProvider getProvider() {
        if (this.provider == null) {
            String db = System.getProperty(DATABASE_ARG);
            this.provider = providers.get(db);
            //fallback to default
            if (this.provider == null) {
                this.provider = providers.get(DEFAULT_DB);
            }

            if (this.provider == null) {
                throw new IllegalStateException("PersistenceManagerProvider is not properly setup. "
                    + "Maybe you misspelled the >-Depp.database< property?");
            }
        }
        return this.provider;
    }

    @Override
    public synchronized void backup(File backupFile) throws Exception {
        getProvider().backup(backupFile);

    }

    @Override
    public void close() {
        if (this.service != null) {
            this.service.close();
            this.service = null;
        }
        getProvider().close();
    }

    @Override
    public void clear() {
        close();
        getProvider().clearFiles();
    }

    @Override
    public void open(File path) throws Exception {
        close();
        getProvider().openFile(path);
    }

}
